/*
 * Copyright © 2012 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Eleni Maria Stea <elenimaria.stea@canonical.com>
 */

#include "sdl_display.h"

#include "mir/graphics/display_configuration.h"

#include <boost/throw_exception.hpp>
#include <stdexcept>

#include <GL/gl.h>

#include <functional>

namespace mgs = mir::graphics::sdl;
namespace mg = mir::graphics;
namespace geom = mir::geometry;

namespace
{
class NullDisplayConfiguration : public mg::DisplayConfiguration {
public:
	void for_each_card(std::function<void(mg::DisplayConfigurationCard const&)>)
    {
    }

    void for_each_output(std::function<void(mg::DisplayConfigurationOutput const&)>)
    {
    }

};
}

mgs::SDLDisplay::SDLDisplay()
{
	if(SDL_Init(SDL_INIT_VIDEO) == -1) {
		BOOST_THROW_EXCEPTION(std::runtime_error("Failed to initialize SDL\n"));
	}

	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 8);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	if(!(sdl_surf = SDL_SetVideoMode(800, 600, 16, SDL_OPENGL | SDL_RESIZABLE))) {
		SDL_Quit();
		BOOST_THROW_EXCEPTION(std::runtime_error(std::string("SDL backend: Couldn't set GL mode: ") + std::string(SDL_GetError()) + std::string("\n")));
	}

	make_current();
}

mgs::SDLDisplay::~SDLDisplay()
{
	SDL_Quit();
}

void mgs::SDLDisplay::make_current()
{
}

void mgs::SDLDisplay::clear()
{
	glClear(GL_COLOR_BUFFER_BIT);
}

bool mgs::SDLDisplay::post_update()
{
	SDL_GL_SwapBuffers();
	return true;
}

geom::Rectangle mgs::SDLDisplay::view_area() const
{
	geom::Point pt {geom::X{0}, geom::Y{0}};
	geom::Size sz {
		geom::Width {sdl_surf->w},
		geom::Height {sdl_surf->h}
	};

	return geom::Rectangle {pt, sz};
}

void mgs::SDLDisplay::for_each_display_buffer(std::function<void(mg::DisplayBuffer&)> const& f)
{
	f(*this);
}

std::shared_ptr<mg::DisplayConfiguration> mgs::SDLDisplay::configuration()
{
	return std::make_shared<NullDisplayConfiguration>();
}
