/*
 * Copyright © 2012 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Eleni Maria Stea <elenimaria.stea@canonical.com>
 */

#ifndef MIR_GRAPHICS_SDL_SDL_DISPLAY_H_
#define MIR_GRAPHICS_SDL_SDL_DISPLAY_H_

#include "mir/graphics/display.h"
#include "mir/graphics/display_buffer.h"
#include "mir/graphics/display_configuration.h"

#include <SDL/SDL.h>
#include <memory>

namespace mir
{
namespace geometry
{
class Rectangle;
}
namespace graphics
{
namespace sdl
{

class SDLDisplay : public Display, public DisplayBuffer {
public:
	SDLDisplay();
	~SDLDisplay();

	/* public API */

	/* from DisplayBuffer */

	void make_current();
	void clear();
	bool post_update();

	/* from Display */

	geometry::Rectangle view_area() const;
	void for_each_display_buffer(std::function<void(DisplayBuffer&)> const& f);
    std::shared_ptr<DisplayConfiguration> configuration();

private:
	SDL_Surface* sdl_surf;
};

}
}
}

#endif /* MIR_GRAPHICS_SDL_SDL_DISPLAY_H_ */
